<?php # $Id: functions_smarty.inc.php,v 1.19 2005/03/14 11:29:05 garvinhicking Exp $
# Copyright (c) 2003-2005, Jannis Hermanns (on behalf the Serendipity Developer Team)
# All rights reserved.  See LICENSE file for licensing details

function &serendipity_smarty_fetch($block, $file, $echo = false) {
    global $serendipity;

    $output = $serendipity['smarty']->fetch('file:'. serendipity_getTemplateFile($file, 'serendipityPath'), null, null, ($echo === true && $serendipity['smarty_raw_mode']));
    $serendipity['smarty']->assign($block, $output);

    return $output;
}

function serendipity_emptyPrefix($string, $prefix = ': ') {
    return (!empty($string) ? $prefix . htmlspecialchars($string) : '');
}

function serendipity_smarty_showPlugin($params, &$smarty) {
    global $serendipity;

    if (!isset($params['class']) && !isset($params['id'])) {
        $smarty->trigger_error(__FUNCTION__ .": missing 'class' or 'id' parameter");
        return;
    }

    if (!isset($params['side'])) {
        $params['side'] = '*';
    }

    if (!isset($params['negate'])) {
        $params['negate'] = null;
    }

    return serendipity_plugin_api::generate_plugins($params['side'], null, $params['negate'], $params['class'], $params['id']);
}

function serendipity_smarty_hookPlugin($params, &$smarty) {
    global $serendipity;
    static $hookable = array('frontend_header',
                             'entries_header',
                             'entries_footer',
                             'frontend_comment');

    if (!isset($params['hook'])) {
        $smarty->trigger_error(__FUNCTION__ .": missing 'hook' parameter");
        return;
    }

    if (!in_array($params['hook'], $hookable) && $params['hookAll'] != 'true') {
        $smarty->trigger_error(__FUNCTION__ .": illegal hook '". $params['hook'] ."'");
        return;
    }

    if (!isset($params['data'])) {
        $params['data'] = &$serendipity;
    }

    if (!isset($params['addData'])) {
        $params['addData'] = null;
    }

    serendipity_plugin_api::hook_event($params['hook'], $params['data'], $params['addData']);
}


function serendipity_smarty_printSidebar($params, &$smarty) {
    if ( !isset($params['side']) ) {
        $smarty->trigger_error(__FUNCTION__ .": missing 'side' parameter");
        return;
    }
    return serendipity_plugin_api::generate_plugins($params['side']);
}

function serendipity_smarty_getFile($params, &$smarty) {
    if ( !isset($params['file']) ) {
        $smarty->trigger_error(__FUNCTION__ .": missing 'file' parameter");
        return;
    }
    return serendipity_getTemplateFile($params['file']);
}


function serendipity_smarty_formatTime($timestamp, $format, $useOffset = true) {
    if (defined($format)) {
        return serendipity_formatTime(constant($format), $timestamp, $useOffset);
    } else {
        return serendipity_formatTime($format, $timestamp, $useOffset);
    }
}

function &serendipity_smarty_printComments($params, &$smarty) {
    global $serendipity;

    if (!isset($params['entry'])) {
        $smarty->trigger_error(__FUNCTION__ .": missing 'entry' parameter");
        return;
    }

    if (!isset($params['mode'])) {
        $params['mode'] = VIEWMODE_THREADED;
    }

    $comments = serendipity_fetchComments($params['entry']);

    if (!empty($serendipity['POST']['preview'])) {
        $comments[] =
            array(
                    'email'     => $serendipity['POST']['email'],
                    'author'    => $serendipity['POST']['name'],
                    'body'      => $serendipity['POST']['comment'],
                    'url'       => $serendipity['POST']['url'],
                    'parent_id' => $serendipity['POST']['replyTo'],
                    'timestamp' => time()
            );
    }

    return serendipity_printComments($comments, $params['mode']);
}

function serendipity_smarty_printTrackbacks($params, &$smarty) {
    if ( !isset($params['entry']) ) {
        $smarty->trigger_error(__FUNCTION__ .": missing 'entry' parameter");
        return;
    }

    return serendipity_printTrackbacks(serendipity_fetchTrackbacks($params['entry']));
}

function serendipity_smarty_init() {
    global $serendipity;

    if (!isset($serendipity['smarty'])) {
        @define('SMARTY_DIR', S9Y_INCLUDE_PATH . 'bundled-libs/Smarty/libs/');
        require_once SMARTY_DIR . 'Smarty.class.php';
        $serendipity['smarty'] = new Smarty;
        if ($serendipity['production'] === 'debug') {
            $serendipity['smarty']->force_compile   = true;
            $serendipity['smarty']->debugging       = true;
        }
        $serendipity['smarty']->template_dir  = $serendipity['serendipityPath'] . $serendipity['templatePath'] . $serendipity['template'];
        $serendipity['smarty']->compile_dir   = $serendipity['serendipityPath'] . PATH_SMARTY_COMPILE;
        $serendipity['smarty']->config_dir    = &$serendipity['smarty']->template_dir;
        $serendipity['smarty']->secure_dir    = array($serendipity['serendipityPath'] . $serendipity['templatePath']);
        $serendipity['smarty']->security_settings['MODIFIER_FUNCS'] = array('sprintf', 'sizeof', 'count');
        $serendipity['smarty']->security      = true;
        $serendipity['smarty']->use_sub_dirs  = false;
        $serendipity['smarty']->compile_check = true;
        $serendipity['smarty']->compile_id    = &$serendipity['template'];

        $serendipity['smarty']->register_modifier('makeFilename', 'serendipity_makeFilename');
        $serendipity['smarty']->register_modifier('xhtml_target', 'serendipity_xhtml_target');
        $serendipity['smarty']->register_modifier('emptyPrefix', 'serendipity_emptyPrefix');
        $serendipity['smarty']->register_modifier('formatTime', 'serendipity_smarty_formatTime');
        $serendipity['smarty']->register_function('serendipity_printSidebar', 'serendipity_smarty_printSidebar');
        $serendipity['smarty']->register_function('serendipity_hookPlugin', 'serendipity_smarty_hookPlugin');
        $serendipity['smarty']->register_function('serendipity_showPlugin', 'serendipity_smarty_showPlugin');
        $serendipity['smarty']->register_function('serendipity_getFile', 'serendipity_smarty_getFile');
        $serendipity['smarty']->register_function('serendipity_printComments', 'serendipity_smarty_printComments');
        $serendipity['smarty']->register_function('serendipity_printTrackbacks', 'serendipity_smarty_printTrackbacks');
    }

    if (!isset($serendipity['smarty_raw_mode'])) {
        if (file_exists($serendipity['smarty']->template_dir . '/layout.php') && $serendipity['template'] != 'default') {
            $serendipity['smarty_raw_mode'] = true;
        } else {
            $serendipity['smarty_raw_mode'] = false;
        }
    }

    if (!isset($serendipity['smarty_file'])) {
        $serendipity['smarty_file'] = 'index.tpl';
    }

    $serendipity['smarty']->assign(
        array(
            'head_charset'              => LANG_CHARSET,
            'head_version'              => $serendipity['version'],
            'head_title'                => $serendipity['head_title'],
            'head_subtitle'             => $serendipity['head_subtitle'],
            'head_link_stylesheet'      => serendipity_rewriteURL('serendipity.css'),

            'CONST'                     => get_defined_constants(),

            'is_xhtml'                  => $serendipity['XHTML11'],
            'use_popups'                => $serendipity['enablePopup'],
            'is_embedded'               => (!$serendipity['embed'] || $serendipity['embed'] === 'false' || $serendipity['embed'] === false) ? false : true,
            'is_raw_mode'               => $serendipity['smarty_raw_mode'],

            'entry_id'                  => (isset($serendipity['GET']['id']) && is_numeric($serendipity['GET']['id'])) ? $serendipity['GET']['id'] : false,
            'is_single_entry'           => (isset($serendipity['GET']['id']) && is_numeric($serendipity['GET']['id'])),

            'blogTitle'                 => htmlspecialchars($serendipity['blogTitle']),
            'blogSubTitle'              => (!empty($serendipity['blogSubTitle']) ? htmlspecialchars($serendipity['blogSubTitle']) : ''),
            'blogDescription'           => htmlspecialchars($serendipity['blogDescription']),

            'serendipityHTTPPath'       => $serendipity['serendipityHTTPPath'],
            'serendipityBaseURL'        => $serendipity['baseURL'],
            'serendipityRewritePrefix'  => $serendipity['rewrite'] == 'none' ? $serendipity['indexFile'] . '?/' : '',
            'serendipityIndexFile'      => $serendipity['indexFile'],
            'serendipityVersion'        => $serendipity['version'],
            
            'dateRange'                 => (!empty($serendipity['range']) ? $serendipity['range'] : array())
        )
    );

    return true;
}

/* Function can be called from foreign applications. ob_start() needs to
  have been called before, and will be parsed into Smarty here */
function serendipity_smarty_shutdown($serendipity_directory = '') {
global $serendipity;

    $cwd = getcwd();
    chdir($serendipity_directory);
    $raw_data = ob_get_contents();
    ob_end_clean();
    $serendipity['smarty']->assign('content_message', $raw_data);

    serendipity_smarty_fetch('CONTENT', 'content.tpl');
    if (empty($serendipity['smarty_file'])) {
        $serendipity['smarty_file'] = '404.tpl';
    }
    $serendipity['smarty']->display(serendipity_getTemplateFile($serendipity['smarty_file'], 'serendipityPath'));
}
